@@ -25,6 +25,7 @@ from omfit_classes.omfit_base import OMFITtmp, OMFITmodule, OMFITtree
 from omfit_classes.omfit_json import OMFITjson, OMFITsettings, SettingsName
 from omfit_classes.omfit_omas_utils import setup_hardware_overlay_cx, add_hardware_to_ods
 from omfit_classes.omfit_formatter import omfit_formatter
+from omfit_classes.utils_fusion import is_device
 
 __all__ = ['efitviewer']
 
@@ -228,10 +229,14 @@ def list_available_hardware_descriptors(device):
     hw0 = 'setup_'
     hw1 = '_hardware_description_'
     available_hardware_d3d = [
-        a.split(hw1 + 'd3d')[0].split(hw0)[-1] for a in omfit_classes.omfit_omas_d3d.__all__ if a.startswith(hw0) and a.endswith(hw1 + 'd3d')
+        a.split(hw1 + 'd3d')[0].split(hw0)[-1]
+        for a in omfit_classes.omfit_omas_d3d.__all__
+        if a.startswith(hw0) and a.endswith(hw1 + 'd3d')
     ]
     available_hardware_east = [
-        a.split(hw1 + 'east')[0].split(hw0)[-1] for a in omfit_classes.omfit_omas_east.__all__ if a.startswith(hw0) and a.endswith(hw1 + 'east')
+        a.split(hw1 + 'east')[0].split(hw0)[-1]
+        for a in omfit_classes.omfit_omas_east.__all__
+        if a.startswith(hw0) and a.endswith(hw1 + 'east')
     ]
     available_hardware_kstar = [
         a.split(hw1 + 'kstar')[0].split(hw0)[-1]
@@ -239,7 +244,9 @@ def list_available_hardware_descriptors(device):
         if a.startswith(hw0) and a.endswith(hw1 + 'kstar')
     ]
     available_hardware_general = [
-        a.split(hw1 + 'general')[0].split(hw0)[-1] for a in omfit_classes.omfit_omas.__all__ if a.startswith(hw0) and a.endswith(hw1 + 'general')
+        a.split(hw1 + 'general')[0].split(hw0)[-1]
+        for a in omfit_classes.omfit_omas.__all__
+        if a.startswith(hw0) and a.endswith(hw1 + 'general')
     ]
 
     if is_device(device, 'DIII-D'):
