@@ -13,10 +13,11 @@ except ImportError as _excp:
         raise
 
 from omfit_classes.omfit_namelist import OMFITnamelist
+from omfit_classes.omfit_ascii import OMFITascii
 import numpy as np
 import fortranformat
 
-__all__ = ['OMFITmhdin', 'OMFITdprobe']
+__all__ = ['OMFITmhdin', 'OMFITdprobe', 'OMFITnstxMHD']
 
 
 class OMFITmhdin(OMFITnamelist):
@@ -32,84 +33,236 @@ class OMFITmhdin(OMFITnamelist):
 
         :param \**kw: keyword arguments passed to OMFITnamelist.load()
         """
-        fformat = {}
-        fformat['FC'] = fortranformat.FortranRecordReader('(4f12.4,2A12)')
-        fformat['OH'] = fortranformat.FortranRecordReader('(5f10.6)')
-        fformat['VESSEL'] = fortranformat.FortranRecordReader('(4f12.4,2A12)')
 
         # load namelist
         self.outsideOfNamelistIsComment = True
         self.noSpaceIsComment = True
-        super(OMFITnamelist, self).load(*args, **kw)
+        super().load(*args, **kw)
 
-        # get text past namelist
-        comment_item = list(self.keys())[-1]
-        if '__comment' not in comment_item:
-            raise ValueError('Not a valid mhdin.dat file')
-        lines = str(self[comment_item])
-        tmp = lines.split('comment')
-        if len(tmp) > 1:
-            lines, comment = tmp
-        else:
-            lines = tmp[0]
-            comment = ''
-        del self[comment_item]
-        lines = lines.split('\n')
-        lines = [line.expandtabs(12) for line in lines if len(line.strip()) and line.strip()[0] != '!']
+        # out-of-namelist format (DIII-D)
+        if 'RF' not in self['IN3']:
+            fformat = {}
+            fformat['FC'] = fortranformat.FortranRecordReader('(4f12.4,2A12)')
+            fformat['OH'] = fortranformat.FortranRecordReader('(5f10.6)')
+            fformat['VESSEL'] = fortranformat.FortranRecordReader('(4f12.4,2A12)')
 
-        # number of elements per section
-        nfc = len(self['IN3'].get('FCTURN', self['IN3'].get('TURNFC', None)))
-        if self['IN3'].get('IECOIL', 0):
-            nve = len(self['IN3']['RSISVS'])
-            ntf = len(lines) - nfc - nve
-        else:
-            ntf = 0
-            nve = len(lines) - nfc - ntf
+            # get text past namelist
+            comment_item = list(self.keys())[-1]
+            if '__comment' not in comment_item:
+                self.clear()
+                self.dynaLoad = True
+                raise ValueError('Not a valid mhdin.dat file')
+            lines = str(self[comment_item])
+            tmp = lines.split('comment')
+            if len(tmp) > 1:
+                lines, comment = tmp
+            else:
+                lines = tmp[0]
+                comment = ''
+            del self[comment_item]
+            lines = lines.split('\n')
+            lines = [line.expandtabs(12) for line in lines if len(line.strip()) and line.strip()[0] != '!']
 
-        # poloidal field coils
-        # R,Z,DR,DZ,skew_angle1,skew_angle2
-        kk = 0
-        self['FC'] = {}
-        for k in range(nfc):
-            lines[kk] += '%12d%12d' % (0, 0)
-            data = fformat['FC'].read(lines[kk])
-            data[4] = '0.0' if not data[4].strip() or float(data[4]) == 0 else data[4]
-            data[5] = '90.0' if not data[5].strip() or float(data[5]) == 0 else data[5]
-            data = np.array(list(map(float, data)))
-            self['FC']['%d' % (k + 1)] = data
-            kk += 1
-        self['FC'] = np.array(self['FC'].values())
+            # number of elements per section
+            nfc = len(self['IN3'].get('FCTURN', self['IN3'].get('TURNFC', None)))
+            if self['IN3'].get('IECOIL', 0):
+                nve = len(self['IN3']['RSISVS'])
+                ntf = len(lines) - nfc - nve
+            else:
+                ntf = 0
+                nve = len(lines) - nfc - ntf
 
-        # ohmic coils
-        # R,Z,DR,DZ,block
-        if not ntf:
-            self['OH'] = np.array([])
-        else:
-            self['OH'] = {}
-            for k in range(ntf):
-                data = fformat['OH'].read(lines[kk])
-                self['OH']['%d' % (k + 1)] = data
-                kk += 1
-            self['OH'] = np.array(self['OH'].values())
-
-        # conducting vessel segments
-        # R,Z,DR,DZ,skew_angle1,skew_angle2
-        if not nve:
-            self['VESSEL'] = np.array([])
-        else:
-            self['VESSEL'] = {}
-            for k in range(nve):
+            # poloidal field coils
+            # R,Z,DR,DZ,skew_angle1,skew_angle2
+            kk = 0
+            self['FC'] = {}
+            for k in range(nfc):
                 lines[kk] += '%12d%12d' % (0, 0)
-                data = fformat['VESSEL'].read(lines[kk])
+                data = fformat['FC'].read(lines[kk])
                 data[4] = '0.0' if not data[4].strip() or float(data[4]) == 0 else data[4]
                 data[5] = '90.0' if not data[5].strip() or float(data[5]) == 0 else data[5]
                 data = np.array(list(map(float, data)))
-                data = np.array(data)
-                self['VESSEL']['%d' % (k + 1)] = data
+                self['FC']['%d' % (k + 1)] = data
                 kk += 1
-            self['VESSEL'] = np.array(self['VESSEL'].values())
+            self['FC'] = np.array(self['FC'].values())
 
-        self['__trailing_comment__'] = 'comment' + comment
+            # ohmic coils
+            # R,Z,DR,DZ,block
+            if not ntf:
+                self['OH'] = np.array([])
+            else:
+                self['OH'] = {}
+                for k in range(ntf):
+                    data = fformat['OH'].read(lines[kk])
+                    self['OH']['%d' % (k + 1)] = data
+                    kk += 1
+                self['OH'] = np.array(self['OH'].values())
+
+            # conducting vessel segments
+            # R,Z,DR,DZ,skew_angle1,skew_angle2
+            if not nve:
+                self['VESSEL'] = np.array([])
+            else:
+                self['VESSEL'] = {}
+                for k in range(nve):
+                    lines[kk] += '%12d%12d' % (0, 0)
+                    data = fformat['VESSEL'].read(lines[kk])
+                    data[4] = '0.0' if not data[4].strip() or float(data[4]) == 0 else data[4]
+                    data[5] = '90.0' if not data[5].strip() or float(data[5]) == 0 else data[5]
+                    data = np.array(list(map(float, data)))
+                    data = np.array(data)
+                    self['VESSEL']['%d' % (k + 1)] = data
+                    kk += 1
+                self['VESSEL'] = np.array(self['VESSEL'].values())
+
+            # uncomment this to translate out-of-namelist format to in-namelist format
+            if False:
+                # create namelist elements
+                self['IN3']['RF'] = self['FC'][:, 0]
+                self['IN3']['ZF'] = self['FC'][:, 1]
+                self['IN3']['WF'] = self['FC'][:, 2]
+                self['IN3']['HF'] = self['FC'][:, 3]
+                self['IN3']['AF'] = self['FC'][:, 4]
+                self['IN3']['AF2'] = self['FC'][:, 5]
+
+                self['IN3']['RVS'] = self['VESSEL'][:, 0]
+                self['IN3']['ZVS'] = self['VESSEL'][:, 1]
+                self['IN3']['WVS'] = self['VESSEL'][:, 2]
+                self['IN3']['HVS'] = self['VESSEL'][:, 3]
+                self['IN3']['AVS'] = self['VESSEL'][:, 5]
+                self['IN3']['AVS2'] = self['VESSEL'][:, 5]
+
+                self['IN3']['RE'] = self['OH'][:, 0]
+                self['IN3']['ZE'] = self['OH'][:, 1]
+                self['IN3']['WE'] = self['OH'][:, 2]
+                self['IN3']['HE'] = self['OH'][:, 3]
+
+            self['__trailing_comment__'] = 'comment' + comment
+
+        # in-namelist format
+        else:
+
+            # initialize namelist
+            for geom in ['R{element}', 'Z{element}', 'W{element}', 'H{element}', 'A{element}', 'A{element}2']:
+                for element in ['E', 'F', 'VS']:
+                    item = geom.format(element=element)
+                    if item not in self['IN3'] and item not in ['AE', 'AE2']:
+                        self['IN3'][item] = np.array([])
+
+            # vessel and PFs are separated
+            if len(self['IN3']['RVS']):
+                self['FC'] = np.array(
+                    [
+                        self['IN3']['RF'],
+                        self['IN3']['ZF'],
+                        self['IN3']['WF'],
+                        self['IN3']['HF'],
+                        self['IN3']['AF'],
+                        self['IN3']['AF2'] + (self['IN3']['AF2'] == 0) * 90,
+                    ]
+                ).T
+                self['OH'] = np.array(
+                    [self['IN3']['RE'], self['IN3']['ZE'], self['IN3']['WE'], self['IN3']['HE'], self['IN3']['HE'] * 0 + 1]
+                ).T
+                self['VESSEL'] = np.array(
+                    [
+                        self['IN3']['RVS'],
+                        self['IN3']['ZVS'],
+                        self['IN3']['WVS'],
+                        self['IN3']['HVS'],
+                        self['IN3']['AVS'],
+                        self['IN3']['AVS2'] + (self['IN3']['AVS2'] == 0) * 90,
+                    ]
+                ).T
+
+            # vessel as PFs
+            else:
+                tmp = np.unique(self['IN3']['FCID'] - 1)
+                pf_ids = tmp[: -len(self['IN3']['VSNAME'])]
+                vessel_ids = tmp[-len(self['IN3']['VSNAME']) :]
+                split = int(np.max(pf_ids)) - 1
+                PF = np.array(
+                    [
+                        self['IN3']['RF'],
+                        self['IN3']['ZF'],
+                        self['IN3']['WF'],
+                        self['IN3']['HF'],
+                        self['IN3']['AF'],
+                        (self['IN3']['AF2'] + (self['IN3']['AF2'] == 0) * 90),
+                    ]
+                ).T
+                self['FC'] = PF[:split, :]
+                self['OH'] = np.array(
+                    [self['IN3']['RE'], self['IN3']['ZE'], self['IN3']['WE'], self['IN3']['HE'], self['IN3']['HE'] * 0 + 1]
+                ).T
+                self['VESSEL'] = PF[split:, :]
+
+                self['IN3']['RVS'] = self['VESSEL'][:, 0]
+                self['IN3']['ZVS'] = self['VESSEL'][:, 1]
+                self['IN3']['WVS'] = self['VESSEL'][:, 2]
+                self['IN3']['HVS'] = self['VESSEL'][:, 3]
+                self['IN3']['AVS'] = self['VESSEL'][:, 4]
+                self['IN3']['AVS2'] = self['VESSEL'][:, 5]
+
+    @dynaSave
+    def save(self, *args, **kw):
+        r"""
+        Save OMFITmhdin file
+
+        :param \*args: arguments passed to OMFITnamelist.save()
+
+        :param \**kw: keyword arguments passed to OMFITnamelist.save()
+        """
+        fformat = {}
+        fformat['FC'] = fortranformat.FortranRecordWriter('(4f12.4,2A12)')
+        fformat['OH'] = fortranformat.FortranRecordWriter('(5f10.6)')
+        fformat['VESSEL'] = fortranformat.FortranRecordWriter('(4f12.4,2A12)')
+
+        mhdin_special = ['FC', 'OH', 'VESSEL', '__trailing_comment__']
+
+        # remove non-namelist components
+        tmp = {}
+        for item in mhdin_special:
+            if item in self:
+                tmp[item] = self[item]
+                del self[item]
+        angle2_special = {}
+        for item in ['AF2', 'AVS2']:
+            if item not in self['IN3']:
+                continue
+            angle2_special[item] = self['IN3'][item].copy()
+            self['IN3'][item][self['IN3'][item] == 90] = 0.0
+        try:
+            # save namelist section
+            super().save(*args, **kw)
+
+            # append non-namelist components
+            txt = ''
+            for system in mhdin_special:
+                if system not in tmp:
+                    continue
+                if system in fformat:
+                    for item in range(tmp[system].shape[0]):
+                        data = tmp[system][item, :].tolist()
+                        if len(data) == 6:
+                            if data[4] == 0:
+                                data[4] = ''
+                            if data[5] == 90:
+                                data[5] = ''
+                        txt += fformat[system].write(data) + '\n'
+                else:
+                    txt += tmp[system]
+            with open(self.filename, 'a') as f:
+                f.write(txt)
+
+        finally:
+            # restore angles
+            for item in angle2_special:
+                self['IN3'][item] = angle2_special
+            # restore non-namelist components
+            for item in mhdin_special:
+                if item in tmp:
+                    self[item] = tmp[item]
 
     @staticmethod
     def plot_coil(data, patch_facecolor='lightgray', patch_edgecolor='black', label=None, ax=None):
@@ -198,9 +351,9 @@ class OMFITmhdin(OMFITnamelist):
 
         # labels
         if label:
-            for k in range(len(x0)):
-                ax.text(x0[k], y0[k], '\n ' + str(k), ha='left', va='top', size=8, color='w', zorder=1000, weight='bold', clip_on=True)
-                ax.text(x0[k], y0[k], '\n ' + str(k), ha='left', va='top', size=8, color='b', zorder=1001, clip_on=True)
+            for k, name in enumerate(self['IN3']['LPNAME']):
+                ax.text(x0[k], y0[k], '\n ' + name, ha='left', va='top', size=8, color='w', zorder=1000, weight='bold', clip_on=True)
+                ax.text(x0[k], y0[k], '\n ' + name, ha='left', va='top', size=8, color='b', zorder=1001, clip_on=True)
 
     def plot_magnetic_probes(self, display=None, colors=None, label=False, ax=None):
         """
@@ -279,37 +432,42 @@ class OMFITmhdin(OMFITnamelist):
         x0 = x0[np.where(y0 != -self.invalid)]
         y0 = y0[np.where(y0 != -self.invalid)]
 
-        # second, detect the negative-length probes and compute the
-        # 90 deg = pi/2 [rad] angle to be exploited as a correction
-        boo = (1 - np.sign(l0)) / 2.0
-        cor = boo * np.pi / 2.0
+        def probe_endpoints(x0, y0, a0, l0):
+            boo = (1 - np.sign(l0)) / 2.0
+            cor = boo * np.pi / 2.0
 
-        # then, compute the two-point arrays to build the partial rogowskis
-        # as segments rather than single points, applying the correction
-        px = x0 - l0 / 2.0 * np.cos(a0 * np.pi / 180.0 + cor)
-        py = y0 - l0 / 2.0 * np.sin(a0 * np.pi / 180.0 + cor)
-        qx = x0 + l0 / 2.0 * np.cos(a0 * np.pi / 180.0 + cor)
-        qy = y0 + l0 / 2.0 * np.sin(a0 * np.pi / 180.0 + cor)
+            # then, compute the two-point arrays to build the partial rogowskis
+            # as segments rather than single points, applying the correction
+            px = x0 - l0 / 2.0 * np.cos(a0 * np.pi / 180.0 + cor)
+            py = y0 - l0 / 2.0 * np.sin(a0 * np.pi / 180.0 + cor)
+            qx = x0 + l0 / 2.0 * np.cos(a0 * np.pi / 180.0 + cor)
+            qy = y0 + l0 / 2.0 * np.sin(a0 * np.pi / 180.0 + cor)
+
+            segx = []
+            segy = []
+            for k in range(len(x0)):
+                segx.append([px[k], qx[k]])
+                segy.append([py[k], qy[k]])
+            return segx, segy
 
         # finally, plot
+        segx, segy = probe_endpoints(x0, y0, a0, l0)
         for k in range(len(x0)):
-            segx = [px[k], qx[k]]
-            segy = [py[k], qy[k]]
             if colors is None:
                 col = 'r'
             else:
                 col = cm(c0[k])
             if l0[k] > 0:
-                ax.plot(segx, segy, '-', lw=2, color=col, zorder=100, alpha=0.75)
+                ax.plot(segx[k], segy[k], '-', lw=2, color=col, zorder=100, alpha=0.75)
                 ax.plot(x0[k], y0[k], '.', color=col, zorder=100, alpha=0.75, mec='none')
             else:
-                ax.plot(segx, segy, '.-', lw=2, color=col, zorder=100, alpha=0.75, mec='none')
+                ax.plot(segx[k], segy[k], '.-', lw=2, color=col, zorder=100, alpha=0.75, mec='none')
 
         # labels
         if label:
-            for k in range(len(x0)):
-                ax.text(x0[k], y0[k], '\n ' + str(k), ha='left', va='top', size=8, color='w', zorder=1000, weight='bold', clip_on=True)
-                ax.text(x0[k], y0[k], '\n ' + str(k), ha='left', va='top', size=8, color='r', zorder=1001, clip_on=True)
+            for k, name in enumerate(self['IN3']['MPNAM2']):
+                ax.text(x0[k], y0[k], '\n ' + name, ha='left', va='top', size=8, color='w', zorder=1000, weight='bold', clip_on=True)
+                ax.text(x0[k], y0[k], '\n ' + name, ha='left', va='top', size=8, color='r', zorder=1001, clip_on=True)
 
     def plot_poloidal_field_coils(self, edgecolor='none', facecolor='orange', label=False, ax=None):
         """
@@ -447,53 +605,6 @@ class OMFITmhdin(OMFITnamelist):
         """
         return self.plotFigure(*args, **kw)
 
-    @dynaSave
-    def save(self, *args, **kw):
-        r"""
-        Save OMFITmhdin file
-
-        :param \*args: arguments passed to OMFITnamelist.save()
-
-        :param \**kw: keyword arguments passed to OMFITnamelist.save()
-        """
-        fformat = {}
-        fformat['FC'] = fortranformat.FortranRecordWriter('(4f12.4,2A12)')
-        fformat['OH'] = fortranformat.FortranRecordWriter('(5f10.6)')
-        fformat['VESSEL'] = fortranformat.FortranRecordWriter('(4f12.4,2A12)')
-
-        mhdin_special = ['FC', 'OH', 'VESSEL', '__trailing_comment__']
-
-        # remove non-namelist components
-        tmp = {}
-        for item in mhdin_special:
-            tmp[item] = self[item]
-            del self[item]
-        try:
-            # save namelist section
-            super(OMFITnamelist, self).save(*args, **kw)
-
-            # append non-namelist components
-            txt = ''
-            for system in mhdin_special:
-                if system in fformat:
-                    for item in range(tmp[system].shape[0]):
-                        data = tmp[system][item, :].tolist()
-                        if len(data) == 6:
-                            if data[4] == 0:
-                                data[4] = ''
-                            if data[5] == 90:
-                                data[5] = ''
-                        txt += fformat[system].write(data) + '\n'
-                else:
-                    txt += tmp[system]
-            with open(self.filename, 'a') as f:
-                f.write(txt)
-
-        finally:
-            # restore non-namelist components
-            for item in mhdin_special:
-                self[item] = tmp[item]
-
     def aggregate_oh_coils(self, index=None, group=None):
         """
         Aggregate selected OH coils into a single coil
@@ -673,11 +784,54 @@ class OMFITmhdin(OMFITnamelist):
                 else:
                     self['IN3'][item] = np.array(self['IN3'][item])
 
+    def pretty_print(self, default_tilt2=0):
+        for item in ['FC', 'OH', 'VESSEL']:
+            print('# ' + '=' * len(item))
+            print('# ' + item)
+            print('# ' + '=' * len(item))
+            for row in range(self[item].shape[0]):
+                start = ' '
+                end = ','
+                if row == 0:
+                    start = '['
+                if row == self[item].shape[0] - 1:
+                    end = ']'
+                tmp = self[item][row, :].copy()
+                if len(tmp) == 6 and tmp[5] == 90:
+                    tmp[5] = default_tilt2
+                print(start + '[' + ', '.join(map(lambda x: repr(x), tmp)) + ']' + end)
+
+        print('# ==========')
+        print('# Flux loops')
+        print('# ==========')
+        print('R_flux_loop = ', end='')
+        print(repr(self['IN3']['RSI']))
+        print('Z_flux_loop = ', end='')
+        print(repr(self['IN3']['ZSI']))
+        print('name_flux_loop = ', end='')
+        print(repr(self['IN3']['LPNAME']))
+
+        print('# ===============')
+        print('# Magnetic probes')
+        print('# ===============')
+        print('R_magnetic = ', end='')
+        print(repr(self['IN3']['XMP2']))
+        print('Z_magnetic = ', end='')
+        print(repr(self['IN3']['YMP2']))
+        print('A_magnetic = ', end='')
+        print(repr(self['IN3']['AMP2']))
+        print('S_magnetic = ', end='')
+        print(repr(self['IN3']['SMP2']))
+        print('name_magnetic = ', end='')
+        print(repr(self['IN3']['MPNAM2']))
+
+        return self
+
 
 class OMFITdprobe(OMFITmhdin):
     @dynaLoad
     def load(self, *args, **kw):
-        super(OMFITdprobe, self).load(*args, **kw)
+        super().load(*args, **kw)
         for item in list(self['IN3'].keys()):
             if item.upper() not in [
                 'RSISVS',
@@ -699,13 +853,88 @@ class OMFITdprobe(OMFITmhdin):
                 del self['IN3'][item]
 
 
+class OMFITnstxMHD(SortedDict, OMFITascii):
+    """
+    OMFIT class to read NSTX MHD device files such as `device01152015.dat`, `diagSpec01152015.dat` and `signals_020916_PF4.dat`
+    """
+
+    def __init__(self, filename, use_leading_comma=None, **kw):
+        r'''
+        OMFIT class to parse NSTX MHD device files
+
+        :param filename: filename
+
+        :param \**kw: arguments passed to __init__ of OMFITascii
+        '''
+        OMFITascii.__init__(self, filename, **kw)
+        SortedDict.__init__(self)
+        self.dynaLoad = True
+
+    @dynaLoad
+    def load(self):
+        with open(self.filename, 'r') as f:
+            lines = f.read().split('\n')
+        lines = [line for line in lines if len(line.strip()) and not line.startswith(';')]
+        for line in lines:
+            line, description = (line + ';').split(';', 1)
+            line = line.split()
+            if line[1] not in self:
+                self[line[1]] = {}
+            piece = len(self[line[1]])
+            # signals.dat
+            if len(line) == 17:
+                self.type = 'signals'
+                self[line[1]][piece] = dict(
+                    zip(
+                        'name type map_var map_index mds_tree read_sig rel_error abs_error sig_thresh use_err scale pri fitwt t_index t_smooth mds_name'.split(),
+                        line,
+                    )
+                )
+            # diagspec.dat
+            elif len(line) == 12:
+                self.type = 'diagspec'
+                self[line[1]][piece] = dict(zip('name type id rc zc wc hc ac ac2 turns div material'.split(), line))
+            # device.dat
+            elif len(line) == 8:
+                self.type = 'device'
+                self[line[1]][piece] = dict(zip('tag type units rc zc pol_ang tor_ang1 tor_ang2'.split(), line))
+            else:
+                raise ValueError(f'Format not recognized {len(line)}')
+            self[line[1]][piece]['description'] = description.strip(';')
+            for item in self[line[1]][piece]:
+                try:
+                    self[line[1]][piece][item] = ast.literal_eval(self[line[1]][piece][item])
+                except (ValueError, SyntaxError):
+                    pass
+
+        # postprocess
+        if self.type == 'signals':
+            mds_tree_map = {'o': 'operations', 'e': 'engineering', 'a': 'activespec', 'r': 'radiatiion', 'E2': 'efit02', '-': 'computed'}
+            for group in self:
+                for item in self[group]:
+                    self[group][item]['mds_tree'] = mds_tree_map[self[group][item]['mds_tree']]
+
+        return self
+
+    def pretty_print(self):
+        '''
+        Print data in file as arrays, as it is needed for a fortran namelist
+        '''
+        for group in self:
+            for item in self[group][0]:
+                print(f"{group.lower()}_{item.split('(')[0].lower()} = ", end='')
+                print([self[group][k][item] for k in self[group]])
+        return self
+
+
 ############################################
 if '__main__' == __name__:
-    test_classes_main_header()
-    tmp = OMFITmhdin(OMFITsrc + '/../modules/EFUND/TEMPLATES/mhdin.dat')
-    tmp.load()
-
     from matplotlib import pyplot
 
-    tmp.plot(label=True)
-    pyplot.show()
+    test_classes_main_header()
+    for machine in ['nstx-u', 'diii-d']:
+        tmp = OMFITmhdin(OMFITsrc + f'/../modules/EFUND/TEMPLATES/mhdin_{machine}.dat')
+        tmp.load()
+        tmp.pretty_print()
+        tmp.plot(label=True)
+        pyplot.show()
