@@ -53,8 +53,7 @@ __all__ = [
     'interpret_signal',
 ]
 
-D3D_mds_server_trename_selector = {'': 'atlas',
-                                   'lbo':'orion'}
+D3D_mds_server_trename_selector = {'': 'atlas', 'lbo': 'orion'}
 
 # dictionary to translate mnemonic names to MDS+ servers stored in MainSettings['SERVER']
 _MDSserverDict = {
@@ -65,26 +64,28 @@ _MDSserverDict = {
     'DIII-D': D3D_mds_server_trename_selector,
     'gat': D3D_mds_server_trename_selector,
     'D3D': D3D_mds_server_trename_selector,
-
-    'mdsplusdev.gat.com':'mdsplus_dev_gat',
-
+    #
+    'mdsplusdev.gat.com': 'mdsplus_dev_gat',
+    #
     'jet_mdsplus': 'jet_mdsplus',
     'mdsplus.jet.efda.org': 'jet_mdsplus',
     'jet': 'jet_mdsplus',
-
+    #
     'transpgrid': 'transpgrid',
     'transpgrid.pppl.gov': 'transpgrid',
     'transp': 'transpgrid',
-
+    #
     'skylark': 'skylark',
     'skylark.pppl.gov': 'skylark',
     'pppl': 'skylark',
     'NSTX': 'skylark',
-
+    'NSTX-U': 'skylark',
+    'NSTXU': 'skylark',
+    #
     'mst_mdsplus': 'mst_mdsplus',
     'dave.physics.wisc.edu': 'mst_mdsplus',
     'mst': 'mst_mdsplus',
-
+    #
     'alcdata': 'alcdata',
     'alcdata.psfc.mit.edu': 'alcdata',
     'Alcator': 'alcdata',
@@ -93,22 +94,22 @@ _MDSserverDict = {
     'MIT': 'alcdata',
     'alcdata': 'alcdata',
     'alcdata-new': 'alcdata',
-
+    #
     'tcvdata': 'tcv_mdsplus',
     'tcvdata.epfl.ch': 'tcv_mdsplus',
     'TCV': 'tcv_mdsplus',
-
+    #
     'EAST': 'mds_ipp_ac_cn',
-
+    #
     'EAST_GA': 'eastdata',
     'EAST_US': 'eastdata',
-
+    #
     'KSTAR': 'kstar_mds',
     'kstar_mds': 'kstar_mds',
-
+    #
     'AUG': 'aug_mdsplus',
     'ASDEX': 'aug_mdsplus',
-    'aug_mdsplus': 'aug_mdsplus'
+    'aug_mdsplus': 'aug_mdsplus',
 }
 
 # from MainSettings['SERVER'] server to MDS+ top tree name
@@ -122,7 +123,7 @@ _topLevelMDStreename = {
     'mds_ipp_ac_cn': 'EAST',
     'mst_mdsplus': 'MST',
     'aug_mdsplus': 'ASDEX',
-    'transpgrid': 'TRANSP'
+    'transpgrid': 'TRANSP',
 }
 
 my_ip_address = [get_ip()]
@@ -162,11 +163,17 @@ def translate_MDSserver(server, treename):
             scoreTable.append([m.ratio(), item, match])
 
         # return best match
-        scores = (list(map(eval, np.array(scoreTable)[:, 0])))
+        scores = list(map(eval, np.array(scoreTable)[:, 0]))
         if max(scores) > 0.80:
             server = scoreTable[np.array(scores).argmax()][2]
         else:
-            raise OMFITexception("Server '" + server + "' was not recognized.\nYou need to add '" + server + "' to the list of servers in OMFIT['MainSettings']['SERVER']")
+            raise OMFITexception(
+                "Server '"
+                + server
+                + "' was not recognized.\nYou need to add '"
+                + server
+                + "' to the list of servers in OMFIT['MainSettings']['SERVER']"
+            )
 
     try:
         server = OMFIT['MainSettings']['SERVER'][server]['MDS_server']
@@ -196,6 +203,7 @@ def tunneled_MDSserver(MDSserver, quiet=True, forceRemote=False):
 
         # find server in list of supported servers
         username, password, server, port = parse_server(MDSserver, default_username='')
+        tunnel = ''
         for item in OMFIT['MainSettings']['SERVER']:
             if isinstance(OMFIT['MainSettings']['SERVER'][item], dict):
                 if 'MDS_server' in OMFIT['MainSettings']['SERVER'][item]:
@@ -215,7 +223,9 @@ def tunneled_MDSserver(MDSserver, quiet=True, forceRemote=False):
         MDSserver_w_tunnel = MDSserver
         if not OMFITaux['MDSserverReachable'][MDSserver0] and tunnel:
             # handle server tunneling (this function does already buffering)
-            username, server, port = setup_ssh_tunnel(assemble_server(username, '', server, port), tunnel, forceTunnel=True, forceRemote=False, allowEmptyServerUsername=True)
+            username, server, port = setup_ssh_tunnel(
+                assemble_server(username, '', server, port), tunnel, forceTunnel=True, forceRemote=False, allowEmptyServerUsername=True
+            )
             if not len(username):
                 MDSserver_w_tunnel = '%s:%s' % (server, port)
             else:
@@ -374,7 +384,9 @@ class OMFITmds(SortedDict, OMFITmdsObjects):
             for sub in path:
                 subpath = subpath + '.' + sub
                 if sub not in tr:
-                    tr[sub] = OMFITmdsValue(server=self.server0, treename=self.treename0, shot=self.shot0, TDI=subpath, caching=self.caching, quiet=self.quiet)
+                    tr[sub] = OMFITmdsValue(
+                        server=self.server0, treename=self.treename0, shot=self.shot0, TDI=subpath, caching=self.caching, quiet=self.quiet
+                    )
                 tr = tr[sub]
 
         # travel to the subtree
@@ -429,15 +441,26 @@ class OMFITmdsConnection(object):
     def mdsvalue(self, treename, shot, what, timeout=None):
         return self.OMFITmdsConnectionBase.mdsvalue(server=self.tunneled_server, treename=treename, shot=shot, what=what, timeout=timeout)
 
-
     def write(self, treename, shot, node, data, error=None, units=None):
-        return self.OMFITmdsConnectionBase.write(server=self.tunneled_server, treename=treename, shot=shot, node=node, data=data, error=error, units=units)
+        return self.OMFITmdsConnectionBase.write(
+            server=self.tunneled_server, treename=treename, shot=shot, node=node, data=data, error=error, units=units
+        )
 
     def write_dataset(self, treename, shot, subtree, xarray_dataset, quantities=None, translator=None):
-        return self.OMFITmdsConnectionBase.write_dataset(server=self.tunneled_server, treename=treename, shot=shot, subtree=subtree, xarray_dataset=xarray_dataset, quantities=quantities, translator=translator)
+        return self.OMFITmdsConnectionBase.write_dataset(
+            server=self.tunneled_server,
+            treename=treename,
+            shot=shot,
+            subtree=subtree,
+            xarray_dataset=xarray_dataset,
+            quantities=quantities,
+            translator=translator,
+        )
 
     def create_model_tree(self, treename, subtree, data, clear_subtree=False):
-        return self.OMFITmdsConnectionBase.create_model_tree(server=self.tunneled_server, treename=treename, subtree=subtree, data=data, clear_subtree=clear_subtree)
+        return self.OMFITmdsConnectionBase.create_model_tree(
+            server=self.tunneled_server, treename=treename, subtree=subtree, data=data, clear_subtree=clear_subtree
+        )
 
     def tcl(self, tcl):
         return self.OMFITmdsConnectionBase.tcl(server=self.tunneled_server, tcl=tcl)
@@ -458,7 +481,6 @@ class OMFITmdsConnectionBaseClass(SortedDict):
 
     def __init__(self, quiet=True, timeout=-1):
         """
-
         :param quiet: bool
             Suppresses some informational print statements about MDSplus connections being opened / used / checked
 
@@ -468,14 +490,16 @@ class OMFITmdsConnectionBaseClass(SortedDict):
         """
         SortedDict.__init__(self, limit=100)
         self.quiet = quiet
-        if compare_version(getattr(MDSplus,'__version__','0.0.0'), '7.84.0') < 0 and timeout != -1:
+        if compare_version(getattr(MDSplus, '__version__', '0.0.0'), '7.84.0') < 0 and timeout != -1:
             printw(
                 'WARNING: Older versions of MDSplus ignore the timeout keyword. Your setting of {} ms probably will '
-                'not be respected by MDSplus version {}'.format(timeout, getattr(MDSplus,'__version__','0.0.0'))
+                'not be respected by MDSplus version {}'.format(timeout, getattr(MDSplus, '__version__', '0.0.0'))
             )
         self.timeout = timeout
 
     def mdsvalue(self, server, treename, shot, what, timeout=None):
+        warnings.filterwarnings('ignore', message=r"tostring\(\) is deprecated\. Use tobytes\(\) instead\.", category=DeprecationWarning)
+
         timeout = self.timeout if timeout is None else timeout
 
         tmp = re.findall('[\'"].*[\'"]', what)
@@ -543,19 +567,36 @@ class OMFITmdsConnectionBaseClass(SortedDict):
                                     if '@' not in signal:
                                         signal = 'jetppf@' + signal
                                     directory, signal = signal.split('@')
-                                    queryResult = mdsplus_data(self[mds_connection_cache_ID].get('ppfuid("' + directory + '");' + pre + 'jet("' + signal + '",' + shot + ')' + post, timeout=timeout))
+                                    queryResult = mdsplus_data(
+                                        self[mds_connection_cache_ID].get(
+                                            'ppfuid("' + directory + '");' + pre + 'jet("' + signal + '",' + shot + ')' + post,
+                                            timeout=timeout,
+                                        )
+                                    )
                                     printd('* Retrieving data ' + pre + 'jet("' + signal + '",' + shot + ')' + post, topic='MDS')
                                 else:
                                     try:
                                         try:
-                                            queryResult = mdsplus_data(self[mds_connection_cache_ID].get(pre + 'ptdata2("' + signal + '",' + shot + ')' + post, timeout=timeout))
-                                            if queryResult is not None and len(queryResult)==1 and queryResult[0]==0:
+                                            queryResult = mdsplus_data(
+                                                self[mds_connection_cache_ID].get(
+                                                    pre + 'ptdata2("' + signal + '",' + shot + ')' + post, timeout=timeout
+                                                )
+                                            )
+                                            if queryResult is not None and len(queryResult) == 1 and queryResult[0] == 0:
                                                 raise Exception('Try local PTDATA')
-                                            printd('* Retrieving remote data ' + pre + 'ptdata2("' + signal + '",' + shot + ')' + post, topic='MDS')
+                                            printd(
+                                                '* Retrieving remote data ' + pre + 'ptdata2("' + signal + '",' + shot + ')' + post,
+                                                topic='MDS',
+                                            )
                                         except Exception as _excp:
                                             try:
-                                                queryResult = mdsplus_data(MDSplus.TdiExecute(pre + 'ptdata2("' + signal + '",' + shot + ')' + post))
-                                                printd('* Retrieving local data ' + pre + 'ptdata2("' + signal + '",' + shot + ')' + post, topic='MDS')
+                                                queryResult = mdsplus_data(
+                                                    MDSplus.TdiExecute(pre + 'ptdata2("' + signal + '",' + shot + ')' + post)
+                                                )
+                                                printd(
+                                                    '* Retrieving local data ' + pre + 'ptdata2("' + signal + '",' + shot + ')' + post,
+                                                    topic='MDS',
+                                                )
                                             except Exception:
                                                 if 'Try local PTDATA' in repr(_excp):
                                                     pass
@@ -565,17 +606,24 @@ class OMFITmdsConnectionBaseClass(SortedDict):
                                     except Exception as _excp:
                                         printd(repr(_excp), topic='MDS')
                                         try:
-                                            queryResult = mdsplus_data(self[mds_connection_cache_ID].get(pre + 'pseudo("' + signal + '",' + shot + ')' + post, timeout=timeout))
+                                            queryResult = mdsplus_data(
+                                                self[mds_connection_cache_ID].get(
+                                                    pre + 'pseudo("' + signal + '",' + shot + ')' + post, timeout=timeout
+                                                )
+                                            )
                                             printd('* Retrieving data ' + pre + 'pseudo("' + signal + '",' + shot + ')' + post, topic='MDS')
                                         except Exception as _excp:
                                             printd(repr(_excp), topic='MDS')
-                                            raise MDSplus.MdsException('Failed to retrieve data from PTDATA: server=`%s` shot=`%s` what=`%s` [ %s ]' % (server, shot, what, repr(_excp)))
+                                            raise MDSplus.MdsException(
+                                                'Failed to retrieve data from PTDATA: server=`%s` shot=`%s` what=`%s` [ %s ]'
+                                                % (server, shot, what, repr(_excp))
+                                            )
                     else:
                         try:
                             printd('* ' + what, topic='MDS')
                             try:
                                 queryResult = mdsplus_data(self[mds_connection_cache_ID].get(what, timeout=timeout))
-                                if queryResult is not None and len(queryResult)==1 and queryResult[0]==0:
+                                if queryResult is not None and len(queryResult) == 1 and queryResult[0] == 0:
                                     raise Exception('Try local PTDATA')
                             except Exception as _excp:
                                 try:
@@ -586,7 +634,9 @@ class OMFITmdsConnectionBaseClass(SortedDict):
                                     else:
                                         raise _excp
                         except Exception:
-                            raise MDSplus.MdsException('Failed to execute operation: server=`%s` treename=`%s` what=`%s`' % (server, treename, what))
+                            raise MDSplus.MdsException(
+                                'Failed to execute operation: server=`%s` treename=`%s` what=`%s`' % (server, treename, what)
+                            )
 
                     printd('* Querying ' + what + "  ->  " + str(type(queryResult)), topic='MDS')
 
@@ -677,7 +727,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
         :param translator: a function that translates nodes from their dataset value to a string shorter than 12 chars
         '''
         if translator is None:
-            translator = lambda x:x
+            translator = lambda x: x
 
         mds_connection_cache_ID = ('write', server, treename, shot)
         if mds_connection_cache_ID not in self:
@@ -815,6 +865,7 @@ class OMFITmdsConnectionBaseClass(SortedDict):
                     raise
         return False
 
+
 _OMFITmdsConnectionBaseObjects = {}
 
 
@@ -851,25 +902,25 @@ class OMFITcaching(SortedDict):
             cachesDir = self.cachesDir
             if cachesDir:
                 directory, filename = os.path.split(os.sep.join([cachesDir] + list(map(str, key))))
-                filename = omfit_hash(filename,10) + '.pkl'
+                filename = omfit_hash(filename, 10) + '.pkl'
                 if os.path.exists(directory + os.sep + filename):
                     try:
                         with open(directory + os.sep + filename, 'rb') as f:
                             value = pickle.load(f)
                     except Exception:
                         os.remove(directory + os.sep + filename)
-                        raise KeyError('%s cache error: %s' % (self.type, key,))
+                        raise KeyError('%s cache error: %s' % (self.type, key))
                     SortedDict.__setitem__(self, key, b2s(value))
                     return b2s(value)
-                raise KeyError('%s cache miss: %s' % (self.type, key,))
+                raise KeyError('%s cache miss: %s' % (self.type, key))
             else:
-                raise KeyError('%s cache miss: %s' % (self.type, key,))
+                raise KeyError('%s cache miss: %s' % (self.type, key))
 
     def __setitem__(self, key, value):
         cachesDir = self.cachesDir
         if cachesDir:
             directory, filename = os.path.split(os.sep.join([cachesDir] + list(map(str, key))))
-            filename = omfit_hash(filename,10) + '.pkl'
+            filename = omfit_hash(filename, 10) + '.pkl'
             if not os.path.exists(directory):
                 try:
                     os.makedirs(directory)
@@ -887,7 +938,9 @@ class OMFITcaching(SortedDict):
             if cachesDir is True:
                 cachesDir = locals().get('OMFITglobaltmpDir', '/tmp/' + os.environ.get('USER', 'user')) + '/OMFITcache/'
             elif cachesDir and not isinstance(cachesDir, str):
-                raise Exception("OMFIT['MainSettings']['SETUP']['MDS_cache_dir'] can only be a string with a directory or `True` or `False`")
+                raise Exception(
+                    "OMFIT['MainSettings']['SETUP']['MDS_cache_dir'] can only be a string with a directory or `True` or `False`"
+                )
         else:
             cachesDir = self._cachesDir
         if cachesDir:
@@ -900,7 +953,7 @@ class OMFITcaching(SortedDict):
             shutil.rmtree(cachesDir, ignore_errors=True)
 
     def __popup_menu__(self):
-        return [['Purge %s cache' % self.type, self.purge], ]
+        return [['Purge %s cache' % self.type, self.purge]]
 
     # End of class OMFITcaching
 
@@ -999,10 +1052,10 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
         self.quiet = quiet
         self.caching = caching
         self.timeout = timeout
-        if compare_version(getattr(MDSplus,'__version__','0.0.0'), '7.84.0') < 0 and timeout != -1:
+        if compare_version(getattr(MDSplus, '__version__', '0.0.0'), '7.84.0') < 0 and timeout != -1:
             printw(
                 'WARNING: Older versions of MDSplus ignore the timeout keyword. Your setting of {} ms probably will '
-                'not be respected by MDSplus version {}'.format(timeout, getattr(MDSplus,'__version__','0.0.0'))
+                'not be respected by MDSplus version {}'.format(timeout, getattr(MDSplus, '__version__', '0.0.0'))
             )
 
     # DATA (can be overridden by subclassing --> _base_data)
@@ -1090,9 +1143,9 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
         if e is not None and e.size and b2s(e[0]) != '':
             data = uarray(data, np.abs(e))
         dims = ['dim_%d' % k for k in range(data.ndim)]
-        clist=[]
-        elist=[]
-        evalid=[]
+        clist = []
+        elist = []
+        evalid = []
 
         for k, c in enumerate(dims):
             clist.append(self.dim_of(k))
@@ -1168,7 +1221,7 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
                 # already connected
                 try:
                     data = mdsplus_data(self.MDSconn[connection_cache_key].get(TDI, timeout=self.timeout))
-                    if self.treename is None and data is not None and len(data)==1 and data[0]==0:
+                    if self.treename is None and data is not None and len(data) == 1 and data[0] == 0:
                         raise Exception('Try local PTDATA')
                     printd('* Retrieving remote data ' + TDI, topic='MDS')
                 except Exception as _excp:
@@ -1199,7 +1252,7 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
         # unique variable identifier on the server side
         if not hasattr(self, 'server0'):
             self.interpret_id()
-        var = omfit_hash(str((self.server0, self.treename0, self.shot0, self.TDI)),10)
+        var = omfit_hash(str((self.server0, self.treename0, self.shot0, self.TDI)), 10)
         mds_connection_cache_ID = ('read', self.server0, self.treename0, self.shot0, var)
 
         # fetch data
@@ -1263,7 +1316,9 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
             self._units_dim_of = {}
             for k in range(len(self._data.shape)):
                 try:
-                    self._units_dim_of[k], building_cache = self._cached_fetch(mds_connection_cache_ID, 'units(dim_of(_s' + var + ',' + str(k) + '))')
+                    self._units_dim_of[k], building_cache = self._cached_fetch(
+                        mds_connection_cache_ID, 'units(dim_of(_s' + var + ',' + str(k) + '))'
+                    )
                 except Exception:
                     pass
 
@@ -1295,7 +1350,9 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
                 for k in range(len(self._error.shape)):
                     self._error_dim_of[k] = None
                     try:
-                        self._error_dim_of[k], building_cache = self._cached_fetch(mds_connection_cache_ID, 'error_of(dim_of(_s' + var + ',' + str(k) + '))')
+                        self._error_dim_of[k], building_cache = self._cached_fetch(
+                            mds_connection_cache_ID, 'error_of(dim_of(_s' + var + ',' + str(k) + '))'
+                        )
                         if self._error_dim_of[k] is not None:
                             self._error_dim_of[k] = np.atleast_1d(self._error_dim_of[k])
                     except Exception:
@@ -1417,7 +1474,14 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
         return values
 
     def __save_kw__(self):
-        return {'server': self.server, 'shot': self.shot, 'treename': self.treename, 'TDI': self.TDI, 'quiet': self.quiet, 'caching': self.caching}
+        return {
+            'server': self.server,
+            'shot': self.shot,
+            'treename': self.treename,
+            'TDI': self.TDI,
+            'quiet': self.quiet,
+            'caching': self.caching,
+        }
 
     def check(self, check_dim_of=0, debug=False):
         """
@@ -1493,6 +1557,8 @@ class OMFITmdsValue(dict, OMFITmdsObjects):
 
         :return: current axes
         '''
+        from matplotlib import pyplot
+
         tmp = self.xarray()
         tmp.plot(*args, **kw)
         coords = list(tmp.coords.keys())
@@ -1735,9 +1801,7 @@ def interpret_signal(server=None, shot=None, treename=None, tdi=None, scratch=No
                 final_units = final_units[:-1].strip()
     printq('   signal interpreter units cleanup finished: {}'.format(repr(final_units)))
 
-    printq(
-        'Done interpreting signal. Shape(y)={}, x range: {},{} {}'.format(np.shape(y), np.nanmin(x), np.nanmax(x), xu)
-    )
+    printq('Done interpreting signal. Shape(y)={}, x range: {},{} {}'.format(np.shape(y), np.nanmin(x), np.nanmax(x), xu))
     return x, y, final_units if final_units is None else final_units.strip(), xu if xu is None else xu.strip()
 
 
