@@ -715,43 +715,46 @@ def OMFITworkDir(root=None, server=''):
     return dir
 
 
-def for_all_modules(doThis='', deploy=False, skip=False):
+def for_all_modules(doThis='', deploy=False, skip=False, these_modules_only=None):
     """
     This is a utility function (to be used by module developers) which can be used to execute the same command on all of the modules
     Note that this script will overwrite the content of your OMFIT tree.
 
     :param doThis: python script to execute.
        In this script the following variables are defined
-       ``module`` contains the reference to the current module being processed,
+       ``root`` contains the reference to the current module being processed,
        ``moduleID`` the moduleID,
-       ``moduleLocation`` the location of the module in the tree
+       ``rootName`` the location of the module in the tree
        ``moduleFile`` the module filename
 
     :param deploy: save the modules back on their original location
 
     :param skip: skip modules that are already in the tree
 
+    :param these_modules_only: list of modules ID to process (useful for development of `doThis`)
+
     :return: None
     """
     moduleList = OMFIT.availableModules(quiet=True)
 
-    for moduleFile in sorted(list(moduleList.keys()), key=lambda x: x.lower()):
+    if these_modules_only is None:
+        these_modules_only = sorted(list(moduleList.keys()), key=lambda x: x.lower())
+    else:
+        these_modules_only = [moduleFile for moduleFile in moduleList if moduleList[moduleFile]['ID'] in these_modules_only]
+
+    for moduleFile in these_modules_only:
         printi('--== PROCESSING: ' + moduleFile + ' ==--')
 
         moduleID = moduleList[moduleFile]['ID']
-        moduleLocation = "OMFIT['%s']" % moduleID
+        rootName = "OMFIT['%s']" % moduleID
 
         if not skip or moduleID not in OMFIT:
 
             OMFIT.loadModule(
-                moduleList[moduleFile]['path'],
-                location=moduleLocation,
-                withSubmodules=False,
-                availableModulesList=moduleList,
-                checkLicense=False,
+                moduleList[moduleFile]['path'], location=rootName, withSubmodules=False, availableModulesList=moduleList, checkLicense=False
             )
 
-            module = eval(moduleLocation)
+            root = eval(rootName)
 
             exec(doThis, globals(), locals())
 
@@ -1730,6 +1733,17 @@ class OMFITpythonTest(OMFITpythonTask):
     """
 
     def __init__(self, filename, **kw):
+        if not os.path.exists(filename):
+            if os.path.dirname(filename) == '':
+                src_dir = f'{os.path.abspath(os.path.dirname(__file__))}/../..'
+                reg_dir = f'{src_dir}/regression/'
+                if os.access(reg_dir, os.W_OK) and not os.path.exists(f'{src_dir}/public'):
+                    filename = reg_dir + filename
+                    kw['modifyOriginal'] = True
+                    if not os.path.exists(filename):
+                        name = 'test_'.join(os.path.basename(filename[:-3]).split('test_')[1:])
+                        with open(filename, 'w') as f:
+                            f.write(self._header.format(name=name))
         OMFITascii.__init__(self, filename, **kw)
         if not os.path.basename(filename).startswith('test_') or not os.path.basename(filename).endswith('.py'):
             raise OMFITexception(
@@ -1739,6 +1753,7 @@ class OMFITpythonTest(OMFITpythonTask):
             with open(self.filename, 'w') as f:
                 name = 'test_'.join(os.path.basename(filename[:-3]).split('test_')[1:])
                 f.write(self._header.format(name=name))
+
         self._done_tidy = False
 
     _header = '''"""
