@@ -207,7 +207,7 @@ def get_rdb_password(server, username=None, password=None, guest=False, guest_fa
                 username, password = userpass[0], ' '.join(userpass[1:])
     if not len(password):
         if guest_fallback_allowed:
-            printq('Password lookup failed for {}. Using guest/public credentials...')
+            printq(f'Password lookup failed for {server}. Using guest/public credentials...')
             return get_public_rdb_login(server)
         raise ValueError(
             '''Use set_rdb_password('{server:}') to encrypt your password for {server:} (translates to {t_server:}).
@@ -696,6 +696,8 @@ If this file is not available to you contact the GA computer group: iris-support
             where2.append('%s=%s' % (k, self._sqlRepr(v[1])))
         query = 'SELECT * from %s where %s' % (table, ' and '.join(where))
         copy_from = self.select(query)[0]
+        if 'userid' in copy_from:
+            copy_from['userid'] = self.username
         for k, v in list(kw.items()):
             copy_from[k] = v[1]
         insert_row = 'INSERT INTO {0}({1}) VALUES ({2});'.format(
@@ -816,6 +818,8 @@ If this file is not available to you contact the GA computer group: iris-support
                     server0, username, rdb_server[0]
                 )
             )
+        self.username = username
+
         # If direct connection fails, use tunneling
         if not OMFITaux['RDBserverReachable'][server] or force_tunnel:
             # Handle server tunneling (this function does already buffering)
